@@ -11,6 +11,7 @@
 #define ISL_DIM_H
 
 #include <isl/ctx.h>
+#include <isl/printer.h>
 
 #if defined(__cplusplus)
 extern "C" {
@@ -77,6 +78,9 @@ __isl_give isl_dim *isl_dim_range(__isl_take isl_dim *dim);
 __isl_give isl_dim *isl_dim_from_range(__isl_take isl_dim *dim);
 struct isl_dim *isl_dim_underlying(struct isl_dim *dim, unsigned n_div);
 
+__isl_give isl_dim *isl_dim_align_params(__isl_take isl_dim *dim1,
+	__isl_take isl_dim *dim2);
+
 int isl_dim_is_wrapping(__isl_keep isl_dim *dim);
 __isl_give isl_dim *isl_dim_wrap(__isl_take isl_dim *dim);
 __isl_give isl_dim *isl_dim_unwrap(__isl_take isl_dim *dim);
@@ -93,6 +97,10 @@ int isl_dim_compatible(struct isl_dim *dim1, struct isl_dim *dim2);
 unsigned isl_dim_size(__isl_keep isl_dim *dim, enum isl_dim_type type);
 unsigned isl_dim_total(struct isl_dim *dim);
 
+__isl_give isl_printer *isl_printer_print_dim(__isl_take isl_printer *p,
+	__isl_keep isl_dim *dim);
+void isl_dim_dump(__isl_keep isl_dim *dim);
+
 #if defined(__cplusplus)
 }
 #endif
