@@ -11,24 +11,18 @@
 #define ISL_DIV_H
 
 #include <isl/dim.h>
-#include <isl/set.h>
+#include <isl/aff_type.h>
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
-struct isl_div {
-	int ref;
-	struct isl_ctx *ctx;
-
-	struct isl_basic_map	*bmap;
-	isl_int			**line;
-};
+struct isl_div;
 typedef struct isl_div isl_div;
 
+isl_ctx *isl_div_get_ctx(__isl_keep isl_div *div);
+
 struct isl_div *isl_div_alloc(struct isl_dim *dim);
-struct isl_div *isl_basic_map_div(struct isl_basic_map *bmap, int pos);
-struct isl_div *isl_basic_set_div(struct isl_basic_set *bset, int pos);
 __isl_give isl_div *isl_div_copy(__isl_keep isl_div *div);
 void isl_div_free(struct isl_div *c);
 
@@ -44,6 +38,8 @@ void isl_div_set_coefficient(struct isl_div *div,
 unsigned isl_div_dim(__isl_keep isl_div *div, enum isl_dim_type type);
 __isl_give isl_div *isl_div_div(__isl_take isl_div *div, int pos);
 
+__isl_give isl_aff *isl_aff_from_div(__isl_take isl_div *div);
+
 #if defined(__cplusplus)
 }
 #endif
