@@ -38,6 +38,10 @@ struct isl_vec *isl_vec_copy(struct isl_vec *vec);
 struct isl_vec *isl_vec_cow(struct isl_vec *vec);
 void isl_vec_free(struct isl_vec *vec);
 
+isl_ctx *isl_vec_get_ctx(__isl_keep isl_vec *vec);
+
+int isl_vec_is_equal(__isl_keep isl_vec *vec1, __isl_keep isl_vec *vec2);
+
 void isl_vec_dump(__isl_keep isl_vec *vec);
 __isl_give isl_printer *isl_printer_print_vec(__isl_take isl_printer *printer,
 	__isl_keep isl_vec *vec);
@@ -52,8 +56,17 @@ __isl_give isl_vec *isl_vec_add(__isl_take isl_vec *vec1,
 __isl_give isl_vec *isl_vec_extend(__isl_take isl_vec *vec, unsigned size);
 __isl_give isl_vec *isl_vec_zero_extend(__isl_take isl_vec *vec, unsigned size);
 
+__isl_give isl_vec *isl_vec_sort(__isl_take isl_vec *vec);
+
 __isl_give isl_vec *isl_vec_read_from_file(isl_ctx *ctx, FILE *input);
 
+__isl_give isl_vec *isl_vec_drop_els(__isl_take isl_vec *vec,
+	unsigned pos, unsigned n);
+__isl_give isl_vec *isl_vec_insert_els(__isl_take isl_vec *vec,
+	unsigned pos, unsigned n);
+__isl_give isl_vec *isl_vec_insert_zero_els(__isl_take isl_vec *vec,
+	unsigned pos, unsigned n);
+
 #if defined(__cplusplus)
 }
 #endif
