@@ -19,17 +19,6 @@
 
 static struct isl_basic_set *uset_convex_hull_wrap_bounded(struct isl_set *set);
 
-static void swap_ineq(struct isl_basic_map *bmap, unsigned i, unsigned j)
-{
-	isl_int *t;
-
-	if (i != j) {
-		t = bmap->ineq[i];
-		bmap->ineq[i] = bmap->ineq[j];
-		bmap->ineq[j] = t;
-	}
-}
-
 /* Return 1 if constraint c is redundant with respect to the constraints
  * in bmap.  If c is a lower [upper] bound in some variable and bmap
  * does not have a lower [upper] bound in that variable, then c cannot
@@ -127,6 +116,19 @@ __isl_give isl_basic_set *isl_basic_set_remove_redundancies(
 		isl_basic_map_remove_redundancies((struct isl_basic_map *)bset);
 }
 
+/* Remove redundant constraints in each of the basic maps.
+ */
+__isl_give isl_map *isl_map_remove_redundancies(__isl_take isl_map *map)
+{
+	return isl_map_inline_foreach_basic_map(map,
+					    &isl_basic_map_remove_redundancies);
+}
+
+__isl_give isl_set *isl_set_remove_redundancies(__isl_take isl_set *set)
+{
+	return isl_map_remove_redundancies(set);
+}
+
 /* Check if the set set is bound in the direction of the affine
  * constraint c and if so, set the constant term such that the
  * resulting constraint is a bounding constraint for the set.
@@ -795,13 +797,6 @@ error:
 	return NULL;
 }
 
-/* Project out final n dimensions using Fourier-Motzkin */
-static struct isl_set *set_project_out(struct isl_ctx *ctx,
-	struct isl_set *set, unsigned n)
-{
-	return isl_set_remove_dims(set, isl_dim_set, isl_set_n_dim(set) - n, n);
-}
-
 static struct isl_basic_set *convex_hull_0d(struct isl_set *set)
 {
 	struct isl_basic_set *convex_hull;
@@ -899,7 +894,7 @@ int isl_basic_set_is_bounded(__isl_keep isl_basic_set *bset)
 
 	if (!bset)
 		return -1;
-	if (isl_basic_set_fast_is_empty(bset))
+	if (isl_basic_set_plain_is_empty(bset))
 		return 1;
 
 	tab = isl_tab_from_recession_cone(bset, 1);
@@ -1039,7 +1034,7 @@ static struct isl_basic_set *modulo_lineality(struct isl_set *set,
 	if (!set || !lin)
 		goto error;
 	lin_dim = total - lin->n_eq;
-	M = isl_mat_sub_alloc(set->ctx, lin->eq, 0, lin->n_eq, 1, total);
+	M = isl_mat_sub_alloc6(set->ctx, lin->eq, 0, lin->n_eq, 1, total);
 	M = isl_mat_left_hermite(M, 0, &U, &Q);
 	if (!M)
 		goto error;
@@ -1672,7 +1667,7 @@ static struct isl_basic_set *common_constraints(struct isl_basic_set *hull,
 
 	total = isl_dim_total(set->dim);
 	for (i = 0; i < set->p[best]->n_ineq; ++i) {
-		constraints[i].c = isl_mat_sub_alloc(hull->ctx,
+		constraints[i].c = isl_mat_sub_alloc6(hull->ctx,
 			set->p[best]->ineq + i, 0, 1, 0, 1 + total);
 		if (!constraints[i].c)
 			goto error;
