@@ -13,6 +13,8 @@
  * ZAC des vignes, 4 rue Jacques Monod, 91893 Orsay, France 
  */
 
+#include <isl/set.h>
+#include <isl/map.h>
 #include <isl/flow.h>
 
 /* A private structure to keep track of a mapping together with
@@ -100,6 +102,11 @@ void isl_access_info_free(__isl_take isl_access_info *acc)
 	free(acc);
 }
 
+isl_ctx *isl_access_info_get_ctx(__isl_keep isl_access_info *acc)
+{
+	return acc ? isl_map_get_ctx(acc->sink.map) : NULL;
+}
+
 /* Add another source to an isl_access_info structure, making
  * sure the "must" sources are placed before the "may" sources.
  * This function may be called at most max_source times on a
@@ -188,7 +195,7 @@ static int access_sort_cmp(const void *p1, const void *p2)
 
 	h1 = isl_map_get_hash(i1->source_map);
 	h2 = isl_map_get_hash(i2->source_map);
-	return h1 - h2;
+	return h1 > h2 ? 1 : h1 < h2 ? -1 : 0;
 }
 
 /* Sort the must source accesses in order of increasing number of shared
@@ -311,7 +318,7 @@ int isl_flow_foreach(__isl_keep isl_flow *deps,
 		return -1;
 
 	for (i = 0; i < deps->n_source; ++i) {
-		if (isl_map_fast_is_empty(deps->dep[i].map))
+		if (isl_map_plain_is_empty(deps->dep[i].map))
 			continue;
 		if (fn(isl_map_copy(deps->dep[i].map), deps->dep[i].must,
 				deps->dep[i].data, user) < 0)
@@ -350,6 +357,11 @@ void isl_flow_free(__isl_take isl_flow *deps)
 	free(deps);
 }
 
+isl_ctx *isl_flow_get_ctx(__isl_keep isl_flow *deps)
+{
+	return deps ? isl_set_get_ctx(deps->must_no_source) : NULL;
+}
+
 /* Return a map that enforces that the domain iteration occurs after
  * the range iteration at the given level.
  * If level is odd, then the domain iteration should occur after
@@ -475,7 +487,7 @@ static int intermediate_sources(__isl_keep isl_access_info *acc,
 	int k, level;
 	int depth = 2 * isl_map_dim(acc->source[j].map, isl_dim_in) + 1;
 
-	if (isl_map_fast_is_empty(temp_rel[j]))
+	if (isl_map_plain_is_empty(temp_rel[j]))
 		return 0;
 
 	for (k = j - 1; k >= 0; --k) {
@@ -498,7 +510,7 @@ static int intermediate_sources(__isl_keep isl_access_info *acc,
 			copy = isl_map_copy(temp_rel[j]);
 			T = last_later_source(acc, copy, j, sink_level, k,
 					      level, &trest);
-			if (isl_map_fast_is_empty(T)) {
+			if (isl_map_plain_is_empty(T)) {
 				isl_set_free(trest);
 				isl_map_free(T);
 				continue;
@@ -588,8 +600,8 @@ static __isl_give isl_map *all_intermediate_sources(
 	for (k = 0; k < acc->n_must; ++k) {
 		int plevel;
 
-		if (isl_map_fast_is_empty(may_rel[k]) &&
-		    isl_map_fast_is_empty(must_rel[k]))
+		if (isl_map_plain_is_empty(may_rel[k]) &&
+		    isl_map_plain_is_empty(must_rel[k]))
 			continue;
 
 		plevel = acc->level_before(acc->source[k].data,
@@ -741,7 +753,7 @@ static __isl_give isl_flow *compute_val_based_dependences(
 	maydo = isl_set_empty_like(mustdo);
 	if (!mustdo || !maydo)
 		goto error;
-	if (isl_set_fast_is_empty(mustdo))
+	if (isl_set_plain_is_empty(mustdo))
 		goto done;
 
 	must_rel = isl_alloc_array(ctx, struct isl_map *, acc->n_must);
@@ -777,8 +789,8 @@ static __isl_give isl_flow *compute_val_based_dependences(
 
 			intermediate_sources(acc, may_rel, j, level);
 
-			if (isl_set_fast_is_empty(mustdo) &&
-			    isl_set_fast_is_empty(maydo))
+			if (isl_set_plain_is_empty(mustdo) &&
+			    isl_set_plain_is_empty(maydo))
 				break;
 		}
 		for (j = j - 1; j >= 0; --j) {
@@ -828,8 +840,8 @@ static __isl_give isl_flow *compute_val_based_dependences(
 							     may_rel[j]);
 		}
 
-		if (isl_set_fast_is_empty(mustdo) &&
-		    isl_set_fast_is_empty(maydo))
+		if (isl_set_plain_is_empty(mustdo) &&
+		    isl_set_plain_is_empty(maydo))
 			break;
 	}
 
@@ -962,7 +974,7 @@ static __isl_give struct isl_sched_info *sched_info_alloc(
 		goto error;
 
 	for (i = 0; i < n; ++i)
-		info->is_cst[i] = isl_map_fast_is_fixed(map, isl_dim_in, i,
+		info->is_cst[i] = isl_map_plain_is_fixed(map, isl_dim_in, i,
 							&info->cst->el[i]);
 
 	return info;
