@@ -57,7 +57,7 @@ static struct isl_vec *interval_sample(struct isl_basic_set *bset)
 	bset = isl_basic_set_simplify(bset);
 	if (!bset)
 		return NULL;
-	if (isl_basic_set_fast_is_empty(bset))
+	if (isl_basic_set_plain_is_empty(bset))
 		return empty_sample(bset);
 	if (bset->n_eq == 0 && bset->n_ineq == 0)
 		return zero_sample(bset);
@@ -636,7 +636,7 @@ static struct isl_vec *sample_bounded(struct isl_basic_set *bset)
 	if (!bset)
 		return NULL;
 
-	if (isl_basic_set_fast_is_empty(bset))
+	if (isl_basic_set_plain_is_empty(bset))
 		return empty_sample(bset);
 
 	dim = isl_basic_set_total_dim(bset);
@@ -963,7 +963,7 @@ __isl_give isl_vec *isl_basic_set_sample_with_cone(
 	total = isl_basic_set_total_dim(cone);
 	cone_dim = total - cone->n_eq;
 
-	M = isl_mat_sub_alloc(bset->ctx, cone->eq, 0, cone->n_eq, 1, total);
+	M = isl_mat_sub_alloc6(bset->ctx, cone->eq, 0, cone->n_eq, 1, total);
 	M = isl_mat_left_hermite(M, 0, &U, NULL);
 	if (!M)
 		goto error;
@@ -1178,7 +1178,7 @@ static struct isl_vec *basic_set_sample(struct isl_basic_set *bset, int bounded)
 		return NULL;
 
 	ctx = bset->ctx;
-	if (isl_basic_set_fast_is_empty(bset))
+	if (isl_basic_set_plain_is_empty(bset))
 		return empty_sample(bset);
 
 	dim = isl_basic_set_n_dim(bset);
