@@ -392,10 +392,10 @@ __isl_give isl_vertices *isl_basic_set_compute_vertices(
 	int level;
 	int init;
 	unsigned nvar;
-	int *selection;
+	int *selection = NULL;
 	int selected;
-	struct isl_tab_undo **snap;
-	isl_mat *facets;
+	struct isl_tab_undo **snap = NULL;
+	isl_mat *facets = NULL;
 	struct isl_vertex_list *list = NULL;
 	int n_vertices = 0;
 	isl_vertices *vertices;
@@ -403,7 +403,7 @@ __isl_give isl_vertices *isl_basic_set_compute_vertices(
 	if (!bset)
 		return NULL;
 
-	if (isl_basic_set_fast_is_empty(bset))
+	if (isl_basic_set_plain_is_empty(bset))
 		return vertices_empty(bset);
 
 	if (bset->n_eq != 0)
@@ -771,7 +771,7 @@ static int has_opposite(struct isl_facet_todo *todo,
 {
 	for (; *list; list = &(*list)->next) {
 		int eq;
-		eq = isl_basic_set_fast_is_equal(todo->bset, (*list)->bset);
+		eq = isl_basic_set_plain_is_equal(todo->bset, (*list)->bset);
 		if (eq < 0)
 			return -1;
 		if (!eq)
