@@ -3,6 +3,7 @@
 #include <ctype.h>
 #include <cloog/isl/cloog.h>
 #include <cloog/isl/backend.h>
+#include <isl/aff.h>
 #include <isl/set.h>
 
 
@@ -19,6 +20,11 @@ CloogConstraint *cloog_constraint_from_isl_constraint(struct isl_constraint *con
 	return (CloogConstraint *)constraint;
 }
 
+isl_constraint *cloog_constraint_to_isl(CloogConstraint *constraint)
+{
+	return (isl_constraint *)constraint;
+}
+
 isl_basic_set *cloog_constraints_set_to_isl(CloogConstraintSet *constraints)
 {
 	return (isl_basic_set *)constraints;
@@ -97,6 +103,35 @@ CloogConstraint *cloog_constraint_set_defining_equality(
 }
 
 
+struct cloog_isl_other {
+	int level;
+	int found;
+	isl_constraint *u;
+	isl_constraint *l;
+};
+
+
+/* Set other->found to 1 if the given constraint involves other->level
+ * and is different from other->u and other->l.
+ */
+static int check_other_constraint(__isl_take isl_constraint *c, void *user)
+{
+	struct cloog_isl_other *other = user;
+	CloogConstraint *cc;
+
+	if (!isl_constraint_is_equal(c, other->l) &&
+	    !isl_constraint_is_equal(c, other->u)) {
+		cc = cloog_constraint_from_isl_constraint(c);
+		if (cloog_constraint_involves(cc, other->level - 1))
+			other->found = 1;
+	}
+
+	isl_constraint_free(c);
+
+	return other->found ? -1 : 0;
+}
+
+
 /* Check if the variable (e) at position level is defined by a
  * pair of inequalities
  *		 <a, i> + -m e +  <b, p> + k1 >= 0
@@ -117,29 +152,26 @@ CloogConstraint *cloog_constraint_set_defining_inequalities(
 {
 	struct isl_constraint *u;
 	struct isl_constraint *l;
-	struct isl_constraint *c;
 	struct cloog_isl_dim dim;
 	struct isl_basic_set *bset;
+	struct cloog_isl_other other;
 
 	bset = cloog_constraints_set_to_isl(constraints);
 	dim = set_cloog_dim_to_isl_dim(constraints, level - 1);
 	if (!isl_basic_set_has_defining_inequalities(bset, dim.type, dim.pos,
 								&l, &u))
 		return cloog_constraint_invalid();
-	for (c = isl_basic_set_first_constraint(isl_basic_set_copy(bset)); c;
-	     c = isl_constraint_next(c)) {
-		if (isl_constraint_is_equal(c, l))
-			continue;
-		if (isl_constraint_is_equal(c, u))
-			continue;
-		*lower = cloog_constraint_from_isl_constraint(c);
-		if (cloog_constraint_involves(*lower, level-1)) {
-			isl_constraint_free(l);
-			isl_constraint_free(u);
-			*lower = NULL;
-			isl_constraint_free(c);
-			return NULL;
-		}
+
+	other.l = l;
+	other.u = u;
+	other.found = 0;
+	other.level = level;
+	isl_basic_set_foreach_constraint(bset, &check_other_constraint, &other);
+	if (other.found) {
+		isl_constraint_free(l);
+		isl_constraint_free(u);
+		*lower = NULL;
+		return NULL;
 	}
 	*lower = cloog_constraint_from_isl_constraint(l);
 	return cloog_constraint_from_isl_constraint(u);
@@ -186,7 +218,7 @@ CloogEqualities *cloog_equal_alloc(int n, int nb_levels, int nb_parameters)
 
 	equal->total_dim = nb_levels - 1 + nb_parameters;
 	equal->n = n;
-	equal->constraints = ALLOCN(CloogConstraintSet *, n);
+	equal->constraints = ALLOCN(isl_constraint *, n);
 	equal->types = ALLOCN(int, n);
 	for (i = 0; i < n; ++i) {
 		equal->constraints[i] = NULL;
@@ -203,12 +235,9 @@ int cloog_equal_total_dimension(CloogEqualities *equal)
 void cloog_equal_free(CloogEqualities *equal)
 {
 	int i;
-	isl_basic_set *bset;
 
-	for (i = 0; i < equal->n; ++i) {
-		bset = cloog_constraints_set_to_isl(equal->constraints[i]);
-		isl_basic_set_free(bset);
-	}
+	for (i = 0; i < equal->n; ++i)
+		isl_constraint_free(equal->constraints[i]);
 	free(equal->constraints);
 	free(equal->types);
 	free(equal);
@@ -239,7 +268,7 @@ static int cloog_constraint_equal_type(CloogConstraint *cc, int level)
 	int i;
 	isl_int c;
 	int type = EQTYPE_NONE;
-	struct isl_constraint *constraint = &cc->isl;
+	struct isl_constraint *constraint = cloog_constraint_to_isl(cc);
     
 	isl_int_init(c);
 	isl_constraint_get_constant(constraint, &c);
@@ -316,18 +345,12 @@ int cloog_equal_type(CloogEqualities *equal, int level)
 void cloog_equal_add(CloogEqualities *equal, CloogConstraintSet *matrix,
 			int level, CloogConstraint *line, int nb_par)
 { 
-	struct isl_basic_set *bset;
-	unsigned nparam;
+	isl_constraint *c;
 	assert(cloog_constraint_is_valid(line));
   
 	equal->types[level-1] = cloog_constraint_equal_type(line, level);
-	bset = isl_basic_set_from_constraint(isl_constraint_copy(&line->isl));
-	nparam = isl_basic_set_n_param(bset);
-	bset = isl_basic_set_extend(bset, nparam,
-				    equal->total_dim - nparam, 0, 0, 0);
-	bset = isl_basic_set_finalize(bset);
-	equal->constraints[level-1] =
-		cloog_constraint_set_from_isl_basic_set(bset);
+	c = cloog_constraint_to_isl(line);
+	equal->constraints[level - 1] = isl_constraint_copy(c);
 }
 
 
@@ -339,10 +362,8 @@ void cloog_equal_add(CloogEqualities *equal, CloogConstraintSet *matrix,
  */
 void cloog_equal_del(CloogEqualities *equal, int level)
 { 
-	isl_basic_set *bset;
-	bset = cloog_constraints_set_to_isl(equal->constraints[level - 1]);
 	equal->types[level-1] = EQTYPE_NONE;
-	isl_basic_set_free(bset);
+	isl_constraint_free(equal->constraints[level - 1]);
 	equal->constraints[level-1] = NULL;
 }
 
@@ -409,7 +430,8 @@ static struct cloog_isl_dim constraint_cloog_dim_to_isl_dim(
 	struct cloog_isl_dim ci_dim;
 
 	for (i = 0; i < 3; ++i) {
-		unsigned dim = isl_constraint_dim(&constraint->isl, types[i]);
+		isl_constraint *c = cloog_constraint_to_isl(constraint);
+		unsigned dim = isl_constraint_dim(c, types[i]);
 		if (pos < dim) {
 			ci_dim.type = types[i];
 			ci_dim.pos = pos;
@@ -430,7 +452,7 @@ static struct clast_expr *div_expr(CloogConstraint *constraint, int pos,
 	struct clast_expr *e = NULL;
 	struct isl_div *div;
 
-	div = isl_constraint_div(&constraint->isl, pos);
+	div = isl_constraint_div(cloog_constraint_to_isl(constraint), pos);
 
 	cloog_int_init(c);
 	for (i = 0, nb_elts = 0; i < dim; ++i) {
@@ -539,40 +561,51 @@ int cloog_constraint_is_upper_bound(CloogConstraint *constraint, int v)
 
 int cloog_constraint_is_equality(CloogConstraint *constraint)
 {
-	return isl_constraint_is_equality(&constraint->isl);
+	return isl_constraint_is_equality(cloog_constraint_to_isl(constraint));
 }
 
-void cloog_constraint_clear(CloogConstraint *constraint)
+CloogConstraintSet *cloog_constraint_set_drop_constraint(
+	CloogConstraintSet *constraints, CloogConstraint *constraint)
 {
-	isl_constraint_clear(&constraint->isl);
+	isl_basic_set *bset;
+	isl_constraint *c;
+
+	bset = cloog_constraints_set_to_isl(constraints);
+	c = cloog_constraint_to_isl(cloog_constraint_copy(constraint));
+	bset = isl_basic_set_drop_constraint(bset, c);
+	return cloog_constraint_set_from_isl_basic_set(bset);
 }
 
 void cloog_constraint_coefficient_get(CloogConstraint *constraint,
 			int var, cloog_int_t *val)
 {
 	struct cloog_isl_dim dim;
+	isl_constraint *c;
 
 	if (!constraint)
 		return;
 
 	dim = constraint_cloog_dim_to_isl_dim(constraint, var);
-	isl_constraint_get_coefficient(&constraint->isl, dim.type, dim.pos, val);
+	c = cloog_constraint_to_isl(constraint);
+	isl_constraint_get_coefficient(c, dim.type, dim.pos, val);
 }
 
 void cloog_constraint_coefficient_set(CloogConstraint *constraint,
 			int var, cloog_int_t val)
 {
 	struct cloog_isl_dim dim;
+	isl_constraint *c;
 
 	assert(constraint);
 
 	dim = constraint_cloog_dim_to_isl_dim(constraint, var);
-	isl_constraint_set_coefficient(&constraint->isl, dim.type, dim.pos, val);
+	c = cloog_constraint_to_isl(constraint);
+	isl_constraint_set_coefficient(c, dim.type, dim.pos, val);
 }
 
 void cloog_constraint_constant_get(CloogConstraint *constraint, cloog_int_t *val)
 {
-	isl_constraint_get_constant(&constraint->isl, val);
+	isl_constraint_get_constant(cloog_constraint_to_isl(constraint), val);
 }
 
 /**
@@ -586,7 +619,7 @@ void cloog_constraint_copy_coefficients(CloogConstraint *constraint,
 	int i;
 	unsigned dim;
 
-	dim = isl_constraint_dim(&constraint->isl, isl_dim_all);
+	dim = cloog_constraint_total_dimension(constraint);
 
 	for (i = 0; i < dim; ++i)
 		cloog_constraint_coefficient_get(constraint, i, dst+i);
@@ -605,7 +638,9 @@ int cloog_constraint_is_valid(CloogConstraint *constraint)
 
 int cloog_constraint_total_dimension(CloogConstraint *constraint)
 {
-	return isl_constraint_dim(&constraint->isl, isl_dim_all);
+	isl_constraint *c;
+	c = cloog_constraint_to_isl(constraint);
+	return isl_constraint_dim(c, isl_dim_all);
 }
 
 
@@ -621,9 +656,11 @@ int cloog_constraint_total_dimension(CloogConstraint *constraint)
 int cloog_constraint_needs_reduction(CloogConstraint *upper, int level)
 {
 	isl_basic_set *bset;
+	isl_constraint *c;
 	struct cloog_isl_dim dim;
 
-	bset = isl_basic_set_from_constraint(isl_constraint_copy(&upper->isl));
+	c = cloog_constraint_to_isl(upper);
+	bset = isl_basic_set_from_constraint(isl_constraint_copy(c));
 	dim = basic_set_cloog_dim_to_isl_dim(bset, level - 1);
 	isl_basic_set_free(bset);
 
@@ -644,11 +681,15 @@ CloogConstraintSet *cloog_constraint_set_for_reduction(CloogConstraint *upper,
 	CloogConstraint *lower)
 {
 	struct isl_basic_set *bset;
+	isl_constraint *c;
 
-	bset = isl_basic_set_from_constraint(isl_constraint_copy(&upper->isl));
-	if (cloog_constraint_is_valid(lower))
+	c = cloog_constraint_to_isl(upper);
+	bset = isl_basic_set_from_constraint(isl_constraint_copy(c));
+	if (cloog_constraint_is_valid(lower)) {
+		c = cloog_constraint_to_isl(lower);
 		bset = isl_basic_set_add_constraint(bset,
-					    isl_constraint_copy(&lower->isl));
+						    isl_constraint_copy(c));
+	}
 	return cloog_constraint_set_from_isl_basic_set(bset);
 }
 
@@ -668,6 +709,27 @@ static int add_constant_term(CloogConstraint *c, void *user)
 	return 0;
 }
 
+
+/* Return an isl_basic_set representation of the equality stored
+ * at position i in the given CloogEqualities.
+ */
+static __isl_give isl_basic_set *equality_to_basic_set(CloogEqualities *equal,
+	int i)
+{
+	isl_constraint *c;
+	isl_basic_set *bset;
+	unsigned nparam;
+	unsigned nvar;
+
+	c = isl_constraint_copy(equal->constraints[i]);
+	bset = isl_basic_set_from_constraint(c);
+	nparam = isl_basic_set_dim(bset, isl_dim_param);
+	nvar = isl_basic_set_dim(bset, isl_dim_set);
+	bset = isl_basic_set_add(bset, isl_dim_set,
+				 equal->total_dim - (nparam + nvar));
+	return bset;
+}
+
 /**
  * Reduce the modulo guard expressed by "constraints" using equalities
  * found in outer nesting levels (stored in "equal").
@@ -681,14 +743,21 @@ static int add_constant_term(CloogConstraint *c, void *user)
  * We first check if there are any equalities we can use.  If not,
  * there is again nothing to reduce.
  * For the actual reduction, we use isl_basic_set_gist, but this
- * function will only perform the reduction we want hear if the
+ * function will only perform the reduction we want here if the
  * the variable that imposes the modulo constraint has been projected
  * out (i.e., turned into an existentially quantified variable).
  * After the call to isl_basic_set_gist, we need to move the
  * existential variable back into the position where the calling
  * function expects it (assuming there are any constraints left).
- * We do this by adding equality between the given dimension and
+ * We do this by adding an equality between the given dimension and
  * the existentially quantified variable.
+ *
+ * If there are no existentially quantified variables left, then
+ * we don't need to add this equality.
+ * If, on the other hand, the resulting basic set involves more
+ * than one existentially quantified variable, then the caller
+ * will not be able to handle the result, so we just return the
+ * original input instead.
  */
 CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 	int level, CloogEqualities *equal, int nb_par, cloog_int_t *bound)
@@ -702,10 +771,12 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 	struct isl_constraint *c;
 	struct isl_div *div;
 	unsigned constraints_dim;
-	int pos;
-	isl_basic_set *bset;
+	unsigned n_div;
+	isl_basic_set *bset, *orig;
+	isl_aff *aff;
 
 	bset = cloog_constraints_set_to_isl(constraints);
+	orig = isl_basic_set_copy(bset);
 	ctx = isl_basic_set_get_ctx(bset);
 	dim = set_cloog_dim_to_isl_dim(constraints, level - 1);
 	assert(dim.type == isl_dim_set);
@@ -715,15 +786,16 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 		isl_basic_set *bset_j;
 		if (equal->types[j] != EQTYPE_EXAFFINE)
 			continue;
-		bset_j = cloog_constraints_set_to_isl(equal->constraints[j]);
+		bset_j = equality_to_basic_set(equal, j);
 		if (!eq)
-			eq = isl_basic_set_copy(bset_j);
+			eq = bset_j;
 		else
-			eq = isl_basic_set_intersect(eq,
-						    isl_basic_set_copy(bset_j));
+			eq = isl_basic_set_intersect(eq, bset_j);
+	}
+	if (!eq) {
+		isl_basic_set_free(orig);
+		return cloog_constraint_set_from_isl_basic_set(bset);
 	}
-	if (!eq)
-		return constraints;
 
 	idim = isl_dim_map_from_set(isl_basic_set_get_dim(bset));
 	id = isl_basic_map_identity(idim);
@@ -735,14 +807,20 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 	eq = isl_basic_set_remove_dims(eq, isl_dim_set, constraints_dim,
 			isl_basic_set_dim(eq, isl_dim_set) - constraints_dim);
 	bset = isl_basic_set_gist(bset, eq);
-	if (isl_basic_set_dim(bset, isl_dim_div) != 1)
+	n_div = isl_basic_set_dim(bset, isl_dim_div);
+	if (n_div > 1) {
+		isl_basic_set_free(bset);
+		return cloog_constraint_set_from_isl_basic_set(orig);
+	}
+	if (n_div < 1) {
+		isl_basic_set_free(orig);
 		return cloog_constraint_set_from_isl_basic_set(bset);
+	}
 
 	div = isl_basic_set_div(isl_basic_set_copy(bset), 0);
-	c = isl_equality_alloc(isl_basic_set_get_dim(bset));
-	c = isl_constraint_add_div(c, div, &pos);
-	isl_constraint_set_coefficient_si(c, isl_dim_set, dim.pos, 1);
-	isl_constraint_set_coefficient_si(c, isl_dim_div, pos, -1);
+	aff = isl_aff_from_div(div);
+	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, dim.pos, -1);
+	c = isl_equality_from_aff(aff);
 	bset = isl_basic_set_add_constraint(bset, c);
 
 	isl_int_set_si(*bound, 0);
@@ -750,18 +828,19 @@ CloogConstraintSet *cloog_constraint_set_reduce(CloogConstraintSet *constraints,
 	cloog_constraint_set_foreach_constraint(constraints,
 						add_constant_term, bound);
 
+	isl_basic_set_free(orig);
 	return cloog_constraint_set_from_isl_basic_set(bset);
 }
 
 CloogConstraint *cloog_constraint_copy(CloogConstraint *constraint)
 {
 	return cloog_constraint_from_isl_constraint(
-		isl_constraint_copy(&constraint->isl));
+		isl_constraint_copy(cloog_constraint_to_isl(constraint)));
 }
 
 void cloog_constraint_release(CloogConstraint *constraint)
 {
-	isl_constraint_free(&constraint->isl);
+	isl_constraint_free(cloog_constraint_to_isl(constraint));
 }
 
 struct cloog_isl_foreach {
@@ -799,8 +878,90 @@ int cloog_constraint_set_foreach_constraint(CloogConstraintSet *constraints,
 
 CloogConstraint *cloog_equal_constraint(CloogEqualities *equal, int j)
 {
-	isl_basic_set *bset;
-	bset = cloog_constraints_set_to_isl(equal->constraints[j]);
-	return cloog_constraint_from_isl_constraint(
-		    isl_basic_set_first_constraint(isl_basic_set_copy(bset)));
+	isl_constraint *c;
+
+	c = isl_constraint_copy(equal->constraints[j]);
+	return cloog_constraint_from_isl_constraint(c);
+}
+
+/* Given a stride constraint on iterator i (specified by level) of the form
+ *
+ *	i = f(outer iterators) + stride * f(existentials)
+ *
+ * extract f as an isl_aff.
+ */
+static isl_aff *extract_stride_offset(__isl_keep isl_constraint *c,
+	int level, CloogStride *stride)
+{
+	int i;
+	isl_dim *dim = isl_constraint_get_dim(c);
+	isl_local_space *ls = isl_local_space_from_dim(dim);
+	isl_aff *offset = isl_aff_zero(ls);
+	isl_int u;
+	unsigned nparam, nvar;
+
+	isl_int_init(u);
+
+	nparam = isl_constraint_dim(c, isl_dim_param);
+	nvar = isl_constraint_dim(c, isl_dim_set);
+
+	for (i = 0; i < nparam; ++i) {
+		isl_constraint_get_coefficient(c, isl_dim_param, i, &u);
+		isl_int_mul(u, u, stride->factor);
+		offset = isl_aff_set_coefficient(offset, isl_dim_param, i, u);
+	}
+	for (i = 0; i < nvar; ++i) {
+		if (i == level - 1)
+			continue;
+		isl_constraint_get_coefficient(c, isl_dim_set, i, &u);
+		isl_int_mul(u, u, stride->factor);
+		offset = isl_aff_set_coefficient(offset, isl_dim_set, i, u);
+	}
+	isl_constraint_get_constant(c, &u);
+	isl_int_mul(u, u, stride->factor);
+	offset = isl_aff_set_constant(offset, u);
+
+	isl_int_clear(u);
+
+	return offset;
+}
+
+/* Update the given lower bound on level such that it satisfies the stride
+ * constraint.  The computation performed here is essentially the same
+ * as that performed in constraint_stride_lower_c.
+ *
+ * We update the constraint
+ *
+ *	a i + f >= 0
+ *
+ * to
+ *
+ *	i >= s * ceil((-f/a - d)/s) + d
+ *
+ * with s the stride and d the offset encoded in the stride constraint.
+ */
+CloogConstraint *cloog_constraint_stride_lower_bound(CloogConstraint *c,
+	int level, CloogStride *stride)
+{
+	isl_constraint *stride_c = cloog_constraint_to_isl(stride->constraint);
+	isl_constraint *bound = cloog_constraint_to_isl(c);
+	isl_aff *offset;
+	isl_aff *lower;
+
+	lower = isl_constraint_get_bound(bound, isl_dim_set, level - 1);
+	isl_constraint_free(bound);
+
+	offset = extract_stride_offset(stride_c, level, stride);
+
+	lower = isl_aff_sub(lower, isl_aff_copy(offset));
+	lower = isl_aff_scale_down(lower, stride->stride);
+	lower = isl_aff_ceil(lower);
+	lower = isl_aff_scale(lower, stride->stride);
+	lower = isl_aff_add(lower, offset);
+	lower = isl_aff_neg(lower);
+	lower = isl_aff_add_coefficient_si(lower, isl_dim_set, level - 1, 1);
+
+	bound = isl_inequality_from_aff(lower);
+
+	return cloog_constraint_from_isl_constraint(bound);
 }
