@@ -7,6 +7,8 @@
 #include <isl/list.h>
 #include <isl/constraint.h>
 #include <isl/div.h>
+#include <isl/ilp.h>
+#include <isl/aff.h>
 
 CloogDomain *cloog_domain_from_isl_set(struct isl_set *set)
 {
@@ -123,14 +125,10 @@ CloogDomain *cloog_domain_simple_convex(CloogDomain *domain)
 {
 	struct isl_basic_set *hull;
 	isl_set *set = isl_set_from_cloog_domain(domain);
-	unsigned dim = isl_set_dim(set, isl_dim_set);
 
 	if (cloog_domain_isconvex(domain))
 		return cloog_domain_copy(domain);
 
-	if (dim == 0)
-		return cloog_domain_convex(domain);
-
 	hull = isl_set_bounded_simple_hull(isl_set_copy(set));
 	return cloog_domain_from_isl_set(isl_set_from_basic_set(hull));
 }
@@ -723,6 +721,11 @@ static int constraint_can_stride(__isl_take isl_constraint *c, void *user)
 	isl_int v;
 	unsigned n_div;
 
+	if (isl_constraint_is_equality(c)) {
+		isl_constraint_free(c);
+		return 0;
+	}
+
 	isl_int_init(v);
 	isl_constraint_get_coefficient(c, isl_dim_set, ccs->level - 1, &v);
 	if (isl_int_is_pos(v)) {
@@ -822,13 +825,14 @@ struct cloog_stride_lower {
 static int constraint_stride_lower(__isl_take isl_constraint *c, void *user)
 {
 	struct cloog_stride_lower *csl = (struct cloog_stride_lower *)user;
-	int i;
 	isl_int v;
-	isl_int t;
 	isl_constraint *bound;
-	isl_div *div;
-	int pos;
-	unsigned nparam, nvar;
+	isl_aff *b;
+
+	if (isl_constraint_is_equality(c)) {
+		isl_constraint_free(c);
+		return 0;
+	}
 
 	isl_int_init(v);
 	isl_constraint_get_coefficient(c, isl_dim_set, csl->level - 1, &v);
@@ -839,40 +843,22 @@ static int constraint_stride_lower(__isl_take isl_constraint *c, void *user)
 		return 0;
 	}
 
-	isl_int_init(t);
+	b = isl_constraint_get_bound(c, isl_dim_set, csl->level - 1);
 
-	nparam = isl_constraint_dim(c, isl_dim_param);
-	nvar = isl_constraint_dim(c, isl_dim_set);
-	bound = isl_inequality_alloc(isl_basic_set_get_dim(csl->bounds));
-	div = isl_div_alloc(isl_basic_set_get_dim(csl->bounds));
-	isl_int_mul(t, v, csl->stride->stride);
-	isl_div_set_denominator(div, t);
-	for (i = 0; i < nparam; ++i) {
-		isl_constraint_get_coefficient(c, isl_dim_param, i, &t);
-		isl_div_set_coefficient(div, isl_dim_param, i, t);
-	}
-	for (i = 0; i < nvar; ++i) {
-		if (i == csl->level - 1)
-			continue;
-		isl_constraint_get_coefficient(c, isl_dim_set, i, &t);
-		isl_div_set_coefficient(div, isl_dim_set, i, t);
-	}
-	isl_constraint_get_constant(c, &t);
-	isl_int_addmul(t, v, csl->stride->offset);
-	isl_div_set_constant(div, t);
+	b = isl_aff_neg(b);
+	b = isl_aff_add_constant(b, csl->stride->offset);
+	b = isl_aff_scale_down(b, csl->stride->stride);
+	b = isl_aff_floor(b);
+	b = isl_aff_scale(b, csl->stride->stride);
+	isl_int_neg(v, csl->stride->offset);
+	b = isl_aff_add_constant(b, v);
+	b = isl_aff_add_coefficient_si(b, isl_dim_set, csl->level - 1, 1);
+
+	bound = isl_inequality_from_aff(b);
 
-	bound = isl_constraint_add_div(bound, div, &pos);
-	isl_int_set_si(t, 1);
-	isl_constraint_set_coefficient(bound, isl_dim_set,
-					csl->level - 1, t);
-	isl_constraint_set_coefficient(bound, isl_dim_div, pos,
-					csl->stride->stride);
-	isl_int_neg(t, csl->stride->offset);
-	isl_constraint_set_constant(bound, t);
 	csl->bounds = isl_basic_set_add_constraint(csl->bounds, bound);
 
 	isl_int_clear(v);
-	isl_int_clear(t);
 	isl_constraint_free(c);
 
 	return 0;
@@ -896,14 +882,15 @@ static int constraint_stride_lower(__isl_take isl_constraint *c, void *user)
 static int constraint_stride_lower_c(__isl_take isl_constraint *c, void *user)
 {
 	struct cloog_stride_lower *csl = (struct cloog_stride_lower *)user;
-	int i;
 	isl_int v;
-	isl_int t, u;
 	isl_constraint *bound;
 	isl_constraint *csl_c;
-	isl_div *div;
-	int pos;
-	unsigned nparam, nvar;
+	isl_aff *d, *b;
+
+	if (isl_constraint_is_equality(c)) {
+		isl_constraint_free(c);
+		return 0;
+	}
 
 	isl_int_init(v);
 	isl_constraint_get_coefficient(c, isl_dim_set, csl->level - 1, &v);
@@ -914,55 +901,27 @@ static int constraint_stride_lower_c(__isl_take isl_constraint *c, void *user)
 		return 0;
 	}
 
-	csl_c = &csl->stride->constraint->isl;
+	csl_c = cloog_constraint_to_isl(csl->stride->constraint);
 
-	isl_int_init(t);
-	isl_int_init(u);
+	d = isl_constraint_get_aff(csl_c);
+	d = isl_aff_drop_dims(d, isl_dim_div, 0, isl_aff_dim(d, isl_dim_div));
+	d = isl_aff_set_coefficient_si(d, isl_dim_set, csl->level - 1, 0);
+	d = isl_aff_scale(d, csl->stride->factor);
 
-	nparam = isl_constraint_dim(c, isl_dim_param);
-	nvar = isl_constraint_dim(c, isl_dim_set);
-	bound = isl_inequality_alloc(isl_basic_set_get_dim(csl->bounds));
-	div = isl_div_alloc(isl_basic_set_get_dim(csl->bounds));
-	isl_int_mul(t, v, csl->stride->stride);
-	isl_div_set_denominator(div, t);
-	for (i = 0; i < nparam; ++i) {
-		isl_constraint_get_coefficient(c, isl_dim_param, i, &t);
-		isl_constraint_get_coefficient(csl_c, isl_dim_param, i, &u);
-		isl_int_mul(u, u, csl->stride->factor);
-		isl_int_addmul(t, v, u);
-		isl_div_set_coefficient(div, isl_dim_param, i, t);
-		isl_int_neg(u, u);
-		isl_constraint_set_coefficient(bound, isl_dim_param, i, u);
-	}
-	for (i = 0; i < nvar; ++i) {
-		if (i == csl->level - 1)
-			continue;
-		isl_constraint_get_coefficient(c, isl_dim_set, i, &t);
-		isl_constraint_get_coefficient(csl_c, isl_dim_set, i, &u);
-		isl_int_mul(u, u, csl->stride->factor);
-		isl_int_addmul(t, v, u);
-		isl_div_set_coefficient(div, isl_dim_set, i, t);
-		isl_int_neg(u, u);
-		isl_constraint_set_coefficient(bound, isl_dim_set, i, u);
-	}
-	isl_constraint_get_constant(c, &t);
-	isl_constraint_get_constant(csl_c, &u);
-	isl_int_mul(u, u, csl->stride->factor);
-	isl_int_addmul(t, v, u);
-	isl_div_set_constant(div, t);
-	isl_int_neg(u, u);
-	isl_constraint_set_constant(bound, u);
+	b = isl_constraint_get_bound(c, isl_dim_set, csl->level - 1);
+
+	b = isl_aff_neg(b);
+	b = isl_aff_add(b, isl_aff_copy(d));
+	b = isl_aff_scale_down(b, csl->stride->stride);
+	b = isl_aff_floor(b);
+	b = isl_aff_scale(b, csl->stride->stride);
+	b = isl_aff_sub(b, d);
+	b = isl_aff_add_coefficient_si(b, isl_dim_set, csl->level - 1, 1);
+
+	bound = isl_inequality_from_aff(b);
 
-	bound = isl_constraint_add_div(bound, div, &pos);
-	isl_int_set_si(t, 1);
-	isl_constraint_set_coefficient(bound, isl_dim_set,
-					csl->level - 1, t);
-	isl_constraint_set_coefficient(bound, isl_dim_div, pos,
-					csl->stride->stride);
 	csl->bounds = isl_basic_set_add_constraint(csl->bounds, bound);
 
-	isl_int_clear(u);
-	isl_int_clear(t);
 	isl_int_clear(v);
 	isl_constraint_free(c);
 
@@ -1011,6 +970,26 @@ CloogDomain *cloog_domain_stride_lower_bound(CloogDomain *domain, int level,
 }
 
 
+/* Add stride constraint, if any, to domain.
+ */
+CloogDomain *cloog_domain_add_stride_constraint(CloogDomain *domain,
+	CloogStride *stride)
+{
+	isl_constraint *c;
+	isl_set *set;
+
+	if (!stride || !stride->constraint)
+		return domain;
+
+	set = isl_set_from_cloog_domain(domain);
+	c = isl_constraint_copy(cloog_constraint_to_isl(stride->constraint));
+
+	set = isl_set_add_constraint(set, c);
+
+	return cloog_domain_from_isl_set(set);
+}
+
+
 /**
  * cloog_domain_lazy_equal function:
  * This function returns 1 if the domains given as input are the same, 0 if it
@@ -1097,6 +1076,40 @@ CloogDomain *cloog_domain_bound_splitter(CloogDomain *dom, int level)
 }
 
 
+/* Check whether the union of scattering functions over all domains
+ * is obviously injective.
+ */
+static int injective_scattering(CloogScatteringList *list)
+{
+	isl_map *map;
+	isl_union_map *umap;
+	int injective;
+	int i = 0;
+	char name[30];
+
+	if (!list)
+		return 1;
+	
+	map = isl_map_copy(isl_map_from_cloog_scattering(list->scatt));
+	snprintf(name, sizeof(name), "S%d", i);
+	map = isl_map_set_tuple_name(map, isl_dim_in, name);
+	umap = isl_union_map_from_map(map);
+
+	for (list = list->next, ++i; list; list = list->next, ++i) {
+		map = isl_map_copy(isl_map_from_cloog_scattering(list->scatt));
+		snprintf(name, sizeof(name), "S%d", i);
+		map = isl_map_set_tuple_name(map, isl_dim_in, name);
+		umap = isl_union_map_add_map(umap, map);
+	}
+
+	injective = isl_union_map_plain_is_injective(umap);
+
+	isl_union_map_free(umap);
+
+	return injective;
+}
+
+
 /**
  * cloog_scattering_lazy_block function:
  * This function returns 1 if the two scattering functions s1 and s2 given
@@ -1105,6 +1118,8 @@ CloogDomain *cloog_domain_bound_splitter(CloogDomain *dom, int level)
  * scatterings are applied are the same.
  * In fact this function answers the question "can I
  * safely consider the two domains as only one with two statements (a block) ?".
+ * A difference of 1 in the final dimension is only allowed if the
+ * entire scattering function is injective.
  * - s1 and s2 are the two domains to check for blocking,
  * - scattering is the linked list of all domains,
  * - scattdims is the total number of scattering dimentions.
@@ -1137,9 +1152,13 @@ int cloog_scattering_lazy_block(CloogScattering *s1, CloogScattering *s2,
 		fixed = isl_set_fast_dim_is_fixed(delta, i, &cst);
 		if (fixed != 1)
 			break;
-		if (i+1 < n_scat && !isl_int_is_zero(cst))
+		if (isl_int_is_zero(cst))
+			continue;
+		if (i + 1 < n_scat)
 			break;
-		if (!isl_int_is_zero(cst) && !isl_int_is_one(cst))
+		if (!isl_int_is_one(cst))
+			break;
+		if (!injective_scattering(scattering))
 			break;
 	}
 	block = i >= n_scat;
@@ -1533,7 +1552,7 @@ static void Euclid(cloog_int_t a, cloog_int_t b,
  * quantified variables and then remove any common factors it has
  * with the coefficient at the given level.
  * The result is the value of the stride and if it is not one,
- * the it is possible to construct a CloogStride.
+ * then it is possible to construct a CloogStride.
  * The constraint leading to the stride is stored in the CloogStride
  * as well a value (factor) such that the product of this value
  * and the coefficient at the given level is equal to -1 modulo the stride.
@@ -1668,3 +1687,150 @@ CloogStride *cloog_domain_list_stride(CloogDomainList *list, int level)
 
 	return data.stride;
 }
+
+struct cloog_can_unroll {
+	int can_unroll;
+	int level;
+	isl_constraint *c;
+	isl_set *set;
+	isl_int *n;
+};
+
+
+/*
+ * Check if the given lower bound can be used for unrolling.
+ * If the lower bound involves any existentially quantified
+ * variables, we currently punt.
+ * Otherwise we compute the maximal value of (i - ceil(l) + 1),
+ * with l the given lower bound and i the iterator identified by level.
+ */
+static int is_valid_unrolling_lower_bound(struct cloog_can_unroll *ccu,
+	__isl_keep isl_constraint *c)
+{
+	unsigned n_div;
+	isl_aff *aff;
+	enum isl_lp_result res;
+
+	n_div = isl_constraint_dim(c, isl_dim_div);
+	if (isl_constraint_involves_dims(c, isl_dim_div, 0, n_div))
+		return 0;
+
+	aff = isl_constraint_get_bound(c, isl_dim_set, ccu->level - 1);
+	aff = isl_aff_ceil(aff);
+	aff = isl_aff_neg(aff);
+	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, ccu->level - 1, 1);
+	res = isl_set_max(ccu->set, aff, ccu->n);
+	isl_aff_free(aff);
+
+	if (res == isl_lp_unbounded)
+		return 0;
+
+	assert(res == isl_lp_ok);
+
+	cloog_int_add_ui(*ccu->n, *ccu->n, 1);
+
+	return 1;
+}
+
+
+/* Check if we can unroll based on the given constraint.
+ * Only lower bounds can be used.
+ * Record it if it turns out to be usable and if we haven't recorded
+ * any other constraint already.
+ */
+static int constraint_can_unroll(__isl_take isl_constraint *c, void *user)
+{
+	struct cloog_can_unroll *ccu = (struct cloog_can_unroll *)user;
+	isl_int v;
+
+	isl_int_init(v);
+	isl_constraint_get_coefficient(c, isl_dim_set, ccu->level - 1, &v);
+	if (isl_int_is_pos(v)) {
+		if (!ccu->c && is_valid_unrolling_lower_bound(ccu, c))
+			ccu->c = isl_constraint_copy(c);
+	}
+	isl_int_clear(v);
+	isl_constraint_free(c);
+
+	return 0;
+}
+
+
+/* Check if we can unroll the domain at the current level.
+ * If the domain is a union, we cannot.  Otherwise, we check the
+ * constraints.
+ */
+static int basic_set_can_unroll(__isl_take isl_basic_set *bset, void *user)
+{
+	struct cloog_can_unroll *ccu = (struct cloog_can_unroll *)user;
+	int r = 0;
+
+	if (ccu->c || !ccu->can_unroll)
+		ccu->can_unroll = 0;
+	else {
+		bset = isl_basic_set_remove_redundancies(bset);
+		r = isl_basic_set_foreach_constraint(bset,
+						&constraint_can_unroll, ccu);
+	}
+	isl_basic_set_free(bset);
+	return r;
+}
+
+
+/* Check if we can unroll the given domain at the given level, and
+ * if so, return the single lower bound in *lb and an upper bound
+ * on the number of iterations in *n.
+ * If we cannot unroll, return 0 and set *lb to NULL.
+ *
+ * We can unroll, if we can identify a lower bound on level
+ * such that the number of iterations is bounded by a constant.
+ */
+int cloog_domain_can_unroll(CloogDomain *domain, int level, cloog_int_t *n,
+	CloogConstraint **lb)
+{
+	isl_set *set = isl_set_from_cloog_domain(domain);
+	struct cloog_can_unroll ccu = { 1, level, NULL, set, n };
+	int r;
+
+	*lb = NULL;
+	r = isl_set_foreach_basic_set(set, &basic_set_can_unroll, &ccu);
+	assert(r == 0);
+	if (!ccu.c)
+		ccu.can_unroll = 0;
+	if (!ccu.can_unroll) {
+		isl_constraint_free(ccu.c);
+		return 0;
+	}
+
+	*lb = cloog_constraint_from_isl_constraint(ccu.c);
+
+	return ccu.can_unroll;
+}
+
+
+/* Fix the iterator i at the given level to l + o,
+ * where l is prescribed by the constraint lb and o is equal to offset.
+ * In particular, if lb is the constraint
+ *
+ *	a i >= f(j)
+ *
+ * then l = ceil(f(j)/a).
+ */
+CloogDomain *cloog_domain_fixed_offset(CloogDomain *domain,
+	int level, CloogConstraint *lb, cloog_int_t offset)
+{
+	isl_aff *aff;
+	isl_set *set = isl_set_from_cloog_domain(domain);
+	isl_constraint *c;
+	isl_constraint *eq;
+
+	c = cloog_constraint_to_isl(lb);
+	aff = isl_constraint_get_bound(c, isl_dim_set, level - 1);
+	aff = isl_aff_ceil(aff);
+	aff = isl_aff_add_coefficient_si(aff, isl_dim_set, level - 1, -1);
+	aff = isl_aff_add_constant(aff, offset);
+	eq = isl_equality_from_aff(aff);
+	set = isl_set_add_constraint(set, eq);
+
+	return cloog_domain_from_isl_set(set);
+}
